package ConfFile;

use strict;
use Carp;
use FileHandle;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT_OK = qw(hash array insert_order);
$VERSION = '0.6.2';

sub new {
    my $class = shift;
    my $self = {};
    
    bless $self, $class;
    if (scalar(@_) != 1) {
	croak "$class: $class->new('/path/to/file')";
    }
    $self->{'Filename'} = shift;
    $self->{'line'} = 0;
    $self->{'CLASS'} = $class;
    $self->{'File'} = new FileHandle;
    if (!($self->{'File'}->open("< $self->{'Filename'}"))) {
	croak "$class: error opening $self->{'Filename'} : $!";
    }
	
    $self->load;
    return $self;
}

sub get {
    my $self = shift;
    my @keys = @_;
    
    if (scalar(@keys) == 1) {
	return $self->{'Info'}->{$keys[0]};
    } elsif (scalar(@keys) == 0) {
	return $self->{'Info'};
    }
    my $key = shift @keys;
    return(&_get($self->{'Info'}->{$key}, @keys));
}

sub _get {
    my ($key, @keys) = @_;

    if (scalar(@keys) == 0) {
	return $key;
    }
    my $nkey = shift @keys;

    if (ref($key)) {
	return (&_get($key->{$nkey}, @keys));
    } else {
	return $key;
    }
}
    
sub load {
    my $self = shift;
    
    $self->{'File'}->seek(0,0); # rewind
    $self->{'Info'} = {};
    $self->{'Info'}->{'Insert_Order'} = "order";
    &_load($self, $self->{'File'}, $self->{'Info'}, "INIT");
}

sub _load {
    my ($self, $fh, $ref, $level) = @_;
    my (@info, @item);
    while (<$fh>) {
	$self->{'line'}++;
#	print "working on line: $_";
	next if /^\#/; #ignore comments
	next if ($_ eq "");
	chomp;
	@info = split('#',$_,2); # eliminate comments after information
	if (/^\s*<\//) { # close out open level
#	    print "closing out level: $level\n";
	    $info[0] =~ tr/\>\<\///d;
	    @item = split(' ',$info[0],2);
	    if ($item[0] ne $level) {
	        croak ("Error in $self->{'Filename'} [line $self->{'line'}]: $item[0] doesn't match $level");
	    }
	    return;
	} elsif (/^\s*</) { # open new level
	    $info[0] =~ tr/><//d;
	    @item = split(' ',$info[0],2);
	    $ref->{'Insert_Order'} .= "\0".$item[0];
	    $ref->{$item[0]}->{'Insert_Order'} .= "\0".$item[1];
	    print "order = $ref->{$item[0]}->{'Insert_Order'}\n";
	    $ref->{$item[0]}->{$item[1]} = {};
	    $ref->{$item[0]}->{$item[1]}->{'Insert_Order'} = "order";
	    print "opening level $item[0]\n";
	    &_load($self, $fh, $ref->{$item[0]}->{$item[1]}, $item[0]);
	} else { # item on this level
	    @item = split(' ',$info[0],2); # allows white space in data
	    next if (length($item[0]) == 0);
	    $ref->{'Insert_Order'} .= "\0".$item[0];
	    print "order = $ref->{'Insert_Order'}\n";
	    $ref->{$item[0]} = $item[1];
	}
    }
}

sub insert_order {
    my (@array, $key, %hash);

    if (ref($_[0])) {
	print "insert_order got a reference to a hash\n";
	%hash = %{$_[0]};
    } else {
	print "insert_order got a hash\n";
	%hash = $_[0];
    }
    if (defined($hash{'Insert_Order'})) {
	print "insert_order found order\n";
	@array = split('\0',$hash{'Insert_Order'});
	shift(@array);
	print "order array = [@array]\n";
    } else {
	@array = keys(%hash);
    }
    return @array;
}

sub array_noref { # give me a hash
    my (@array, $key, %hash);

    if (ref($_[0])) {
	print "array got a reference to a hash\n";
	%hash = %{$_[0]};
    } else {
	print "array got a hash\n";
	%hash = $_[0];
    }
    foreach $key (keys(%hash)) {
	next if ref($key);
	next if $key eq "Insert_Order";
	push @array, $key, $hash{$key};
    }
    return @array;
}

sub array { # give me a hash
    my (@array, $key, %hash);

    if (ref($_[0])) {
	print "array got a reference to a hash\n";
	%hash = %{$_[0]};
    } else {
	print "array got a hash\n";
	%hash = $_[0];
    }
    foreach $key (keys(%hash)) {
	next if $key eq "Insert_Order";
	push @array, $key, $hash{$key};
    }
    return @array;
}

sub hash(@) { # give me an array
    my (%hash, $index, @array);

    if (ref($_[0])) {
	print "hash got a reference to an array\n";
	@array = @{$_[0]};
    } else {
	@array = @_;
    }
    $hash{'Insert_Order'} = "order";
    for ($index = 0; $index <= $#array; $index += 2) {
	$hash{$array[$index]} = $array[$index+1];
	$hash{'Insert_Order'} .= "\0".$index;
    }
    return %hash;
}
    
1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

ConfFile - Configuration File parser loads configuration information from a file and into a hash.

=head1 SYNOPSIS

  use ConfFile qw(insert_order array);

  $conf = ConfFile->new("/path/to/configuration/file");
  $item = $conf->get("Domain", "bob.com");
  @items = insert_order($item);
  %hash = hash(@items);
  @array = array(%hash);
  @array = array_noref(%hash);


=head1 DESCRIPTION

This module loads an apache style configuration file into a multi-level hash and provides routines to pull the information back out (even in the order it was put into the hash).

<Domain bob.com>
    Server  server.bob.com
    OS      Solaris-2.5.1
    Arch    sparc
    <IMAP pop.bob.com>
            Login  bob
            Pass   bobpass
            Quota  10000
            ACL    lrws
    </IMAP>
</Domain>

get takes an array or multiple arguements that are the references to what information you would like.

$bob = $conf->get("Domain", "bob.com"); <- this will return a refence to the hash that contains bob.com information

$bob->{'Server'} contains server.bob.com
$bob->{'OS'} contains Solaris-2.5.1
$bob->{'IMAP'} contains a reference to the hash that has a hash for pop.bob.com in it.

$pop = $bob->{'IMAP'};
$pop->{'pop.bob.com'}->{'Login'} contains bob
$pop->{'pop.bob.com'}->{'Pass'} contains bobpass

$pass = $conf->get("Domain", "bob.com", "IMAP", "pop.bob.com", "Pass"); <- returns bobpass

insert_order returns the entries in a hash (created by ConfFile) as an array in the order they were put into the hash (by ConfFile).

hash creates hash from an array, it will contain insert_order information
array creates an array from a hash
array_noref creates an array from a hash and leaves out references

insert_order, hash, array, array_noref are all EXPORT_OK'd

=head1 AUTHOR

Eric Estabrooks, estabroo@ispn.com

=head1 SEE ALSO

perl(1).

=cut
